/*** Copyright: OGIHARA Takeshi, Feb. 2011 ***/

#import "FunctionCtrl.h"
#import <Cocoa/Cocoa.h>
#import "QuadraticView.h"
#import "NSControl+AddingCoval.h"
#import "Solver.h"

@implementation FunctionCtrl

- (void)dealloc
{
    [window release];
    [super dealloc];
}

/* Local Method */
- (void)setupMainMenu
{
    NSMenuItem *item;
    NSMenu *menu, *submenu;

    menu = [[[NSMenu alloc] initWithTitle: AppName] autorelease];
    item = [menu addItemWithTitle:@"Action"
        action:@selector(submenuAction:) keyEquivalent:@""];
    submenu = [[[NSMenu alloc] initWithTitle:@"Action"] autorelease];
    [menu setSubmenu:submenu forItem:item];
    item = [submenu addItemWithTitle:@"Reset"
        action:@selector(reset:) keyEquivalent:@"r"];
    [item setTarget:self];
    (void)[submenu addItemWithTitle:@"Quit " AppName
        action:@selector(terminate:) keyEquivalent:@"q"];
    // if target is nil, the message is sent to the first responder.
    [NSApp setMainMenu:menu];
}

static coeffs coef_abc = { 1.0, 0.0, -1.0 };

/* Local Method */
- (void)windowSetUp
{
    const CGFloat space = 32.0;
    const CGFloat vmgn1 = 24.0;
    const CGFloat vmgn2 = vmgn1 + space * 4 + 12.0;
    const CGFloat hmgn1 = 30.0, hmgn2 = 10.0;
    const CGFloat titwid = 16.0, digwid = 48.0, sliderwid = 300.0;
    const CGFloat rootwid = 300.0;
    const int maxtick = 6;
    static NSString *title[] = { @"a", @"b", @"c" };
    NSSize vsize = NSMakeSize(500.0, 500.0);
    NSRect winrect, r;
    NSView *cview;
    NSTextField *tx[3], *t;
    NSSlider *sl[3];
    QuadraticView *qv;
    int i;
    CGFloat vp, hp;
    coeffs_coval_t *qvcov;
    double_coval_t *txcov, *slcov;

    NSNumberFormatter *nFormatter = [[NSNumberFormatter alloc] init];
    [nFormatter autorelease];
    [nFormatter setFormat:@"0.00"];

    winrect.origin = NSMakePoint(200.0, 200.0);
    winrect.size = NSMakeSize(vsize.width, vsize.height + vmgn2);
    window = [[NSWindow alloc] initWithContentRect:winrect
        styleMask: NSTitledWindowMask
        backing:NSBackingStoreBuffered defer:YES];
    [window setReleasedWhenClosed:NO];
    [window setTitle: AppName];
    r.size = vsize;
    r.origin = NSMakePoint(0.0, vmgn2);
    qv = [[[QuadraticView alloc] initWithFrame:r] autorelease];
    cview = [window contentView];

    r = NSMakeRect(hmgn1 + 20.0, vmgn1, rootwid, 16.0);
    t = [[[NSTextField alloc] initWithFrame:r] autorelease];
    [cview addSubview: t];
    [t setEditable:NO];
    [t setBezeled:NO];
    sol = [[Solver alloc] init];
    [sol setTextfield:t];

    vp = vmgn1 + space * 3;
    for (i = 0; i < 3; i++, vp -= space) {
        hp = hmgn1;
        r = NSMakeRect(hp, vp, titwid, 16.0);
        t = [[[NSTextField alloc] initWithFrame:r] autorelease];
        [cview addSubview: t];
        [t setStringValue:title[i]];
        [t setEditable:NO];
        [t setBezeled:NO];
        [t setDrawsBackground:NO];
        hp += titwid + hmgn2;
        r = NSMakeRect(hp, vp-6.0, digwid, 24.0);
        tx[i] = [[[NSTextField alloc] initWithFrame:r] autorelease];
        [cview addSubview: tx[i]];
        [tx[i] setEditable:YES];
        [[tx[i] cell] setSendsActionOnEndEditing:YES];
        [tx[i] setBezeled:YES];
        [tx[i] setDrawsBackground:YES];
        [tx[i] setAlignment: NSRightTextAlignment];
        [tx[i] setFormatter: nFormatter];
	txcov = [tx[i] doubleCoval];

        hp += digwid + hmgn2;
        r = NSMakeRect(hp, vp-6.0, sliderwid, 24.0);
        sl[i] = [[[NSSlider alloc] initWithFrame:r] autorelease];
        [cview addSubview: sl[i]];
        [sl[i] setMaxValue:(float) maxtick];
        [sl[i] setMinValue:(float)-maxtick];
        [sl[i] setNumberOfTickMarks: maxtick * 2 + 1];
        slcov = [sl[i] doubleCoval];
        BindCoval(txcov, slcov);
    }
    qvcov = [qv covalOfCoefficients];
    CovalBridgeInit(double, double, &cov_a);
    CovalBridgeInit(double, double, &cov_b);
    CovalBridgeInit(double, double, &cov_c);
    BridgeCovalMember(&cov_a, 0, qvcov, a);
    BridgeCovalMember(&cov_b, 0, qvcov, b);
    BridgeCovalMember(&cov_c, 0, qvcov, c);
    txcov = [tx[0] doubleCoval];
    BridgeCoval(&cov_a, 1, txcov);
    txcov = [tx[1] doubleCoval];
    BridgeCoval(&cov_b, 1, txcov);
    txcov = [tx[2] doubleCoval];
    BridgeCoval(&cov_c, 1, txcov);
    SetCovalCallSelf(qvcov, YES);
    BindCoval(qvcov, [sol covalOfCoefficients]);

    BindCoval(qvcov, &model_cov);
    CovalAssign(&model_cov, coef_abc);
    // [qv setCoefficients: coef_abc];
    [cview addSubview: qv];
}

- (void)reset:(id)sender
{
    CovalAssign(&model_cov, coef_abc);
}

/* Application Delegate Message */
- (void)applicationDidFinishLaunching:(NSNotification *)note
{
    [self setupMainMenu];
    [self windowSetUp];
    [window makeKeyAndOrderFront:self];
}

@end
