/*** Copyright: OGIHARA Takeshi, Feb. 2011 ***/

#import "NSControl+AddingCoval.h"
#import <objc/runtime.h>

@interface DoubleCovalProxy : NSObject
{
	double_coval_t	myCoval;
}
- (id)initWithControl:(NSControl *)obj double:(double)val;
- (double_coval_t *)coval;
@end

@implementation DoubleCovalProxy

static int set_double(double_coval_t *cov, NSControl *cntrl)
{
	double v = CovalValue(cov);
	[cntrl setDoubleValue:v];
	return COVAL_PROC_SUCCESS;
}

- (void)notifyChange:(id)sender
{
	double v = [sender doubleValue];
	CovalAssign(&myCoval, v);
}

- (id)initWithControl:(NSControl *)obj double:(double)val
{
	if ((self = [super init]) != nil) {
		CovalAssign(&myCoval, val);
		[obj setTarget:self];
		[obj setAction:@selector(notifyChange:)];
		SetCovalCallback(&myCoval, set_double, obj);
	}
	return self;
}

- (void)dealloc {
	UnbindCoval(&myCoval);
	[super dealloc];
}

- (double_coval_t *)coval { return &myCoval; }

@end

@implementation NSControl (AddingCoval)

static char sKey;  // Key address

- (double_coval_t *)doubleCoval
{
	DoubleCovalProxy *proxy = objc_getAssociatedObject(self, &sKey);
	if (proxy == nil) {
		proxy = [[DoubleCovalProxy alloc]
			initWithControl:self double:[self doubleValue]];
		objc_setAssociatedObject(self, &sKey, proxy,
			OBJC_ASSOCIATION_RETAIN);
		// associated object is released automatically.
	}
	return [proxy coval];
}

@end
