/*** Copyright: OGIHARA Takeshi, Jan. 2011 ***/

#import "QuadraticView.h"
#import "QuadraticView+Draw.h"
#import <Cocoa/Cocoa.h>

@implementation QuadraticView

static int coefficientsChanged(coeffs_coval_t *cp, QuadraticView *me)
{
    [me redrawGraph];
    [me display];
    return COVAL_PROC_SUCCESS;
}

- (id)initWithFrame:(NSRect)rect
{
    if ((self = [super initWithFrame:rect]) == nil)
        return nil;
    if ((cache = [[NSImage alloc] initWithSize:rect.size]) == nil) {
        [self release];
        return nil;
    }
    SetCovalCallback(&coefficients, coefficientsChanged, self);
    gridArray = nil;
    return self;
}

- (void)dealloc
{
    UnbindCoval(&coefficients);
    [cache release];
    [gridArray release];
    [super dealloc];
}

- (void)setCoefficients:(coeffs)vals {
	CovalAssign(&coefficients, vals);
}

- (coeffs_coval_t *)covalOfCoefficients { return &coefficients; }

- (void)drawRect:(NSRect)aRect
{
    [cache compositeToPoint:(aRect.origin)
                      fromRect:aRect operation:NSCompositeCopy];
}

- (void)moveGraph:(NSPoint)d  /* move the graph */
{
    double a = CovalMemberValue(&coefficients, a);
    double b = CovalMemberValue(&coefficients, b);
    double c = CovalMemberValue(&coefficients, c);
    // new b := b - 2ax,  new c := c + ax^2 - bx + y
    double w = a * d.x;
    double newb = b - 2.0 * w;
    double newc = c + (w - b) * d.x + d.y;
    CovalAssignMembers2(&coefficients, b, newb, c, newc);
}

- (void)mouseDown:(NSEvent *)event 
{
    /* Get mouse dragging and move the graph */
    NSPoint p, prev, delta;
    const NSUInteger DRAG_MASK = (NSLeftMouseUpMask|NSLeftMouseDraggedMask);
    double scale = (XY_MAX * 2.0) / [self frame].size.width;

    p = [event locationInWindow];
    prev = [self convertPoint:p fromView:nil]; /* View based point */
    for ( ; ; ) {
        event = [[self window] nextEventMatchingMask:DRAG_MASK];
        if ([event type] == NSLeftMouseUp)
            break;
        p = [event locationInWindow];
        p = [self convertPoint:p fromView:nil];
        delta.x = scale * (p.x - prev.x);
        delta.y = scale * (p.y - prev.y);
        [self moveGraph: delta];
        prev = p;
    }
}

@end
